@@ -1,15 +1,16 @@
 import datetime
-import factory
-import random
-import uuid
 import zlib
 
+import factory
+
+from django.contrib.gis import geos
 from django.contrib.gis.geos.geometry import GEOSGeometry
-from django.contrib.gis.geos.point import Point
+from django.utils import timezone
 import pytz
 
 from . import models
 from . import enums
+from . import utils
 
 
 district10 = GEOSGeometry(
@@ -549,26 +550,18 @@ district10 = GEOSGeometry(
 )
 
 
-def get_random_point():
-    """Returns a random point in district10
-    """
-    while True:
-        point = Point(
-            random.uniform(-118.39091357778655, -118.28229399983276),
-            random.uniform(34.00363399975826, 34.07988124764408),
-        )
-        if district10.contains(point):
-            break
-    return point
-
-
 class Provider(factory.DjangoModelFactory):
     class Meta:
         model = models.Provider
+        django_get_or_create = ["name"]
 
     name = factory.Iterator(
         ["Lime", "BlueLA", "Metro Bike", "LongProviderNameCompanyLtdSarlGmoLgbtq"]
     )
+    base_api_url = "http://provider"
+    api_configuration = {"trailing_slash": False}
+    api_authentication = {"type": "none"}
+    agency_api_authentication = {"type": "none"}
 
 
 class Device(factory.DjangoModelFactory):
@@ -578,67 +571,129 @@ class Device(factory.DjangoModelFactory):
     provider = factory.SubFactory(Provider)
     identification_number = factory.Sequence(str)
     model = factory.Iterator(["bicycle-A", "car-B"])
-    category = factory.Iterator(choice[0] for choice in enums.DEVICE_CATEGORY_CHOICES)
-    propulsion = factory.Iterator(
-        choice[0] for choice in enums.DEVICE_PROPULSION_CHOICES
-    )
-    properties = {}
+    category = factory.Iterator(choice.name for choice in enums.DEVICE_CATEGORY)
+    propulsion = factory.Iterator([choice.name] for choice in enums.DEVICE_PROPULSION)
+
+    dn_gps_point = factory.LazyFunction(lambda: utils.get_random_point(district10))
+    dn_gps_timestamp = factory.LazyFunction(timezone.now)
+    dn_status = factory.Iterator(choice.name for choice in enums.DEVICE_STATUS)
 
 
 class Area(factory.DjangoModelFactory):
     class Meta:
         model = models.Area
 
-    creation_date = datetime.datetime(2018, 8, 1, tzinfo=pytz.utc)
-    deletion_date = None
     label = ""
 
+    @factory.post_generation
+    def polygons(self, create, extracted, **kwargs):
+        if not create:
+            return  # build, can't record many to many relationship
+        if extracted:
+            for polygon in extracted:
+                self.polygons.add(polygon)
+        else:
+            geom = geos.Polygon(
+                ((0.0, 0.0), (0.0, 50.0), (50.0, 50.0), (50.0, 0.0), (0.0, 0.0))
+            )
+            poly = Polygon(geom=geom)
+            self.polygons.add(poly)
+
+    @factory.post_generation
+    def providers(self, create, extracted, **kwargs):
+        if not create:
+            return  # build, can't record many to many relationship
+        if extracted:
+            for provider in extracted:
+                self.providers.add(provider)
+        else:
+            self.providers.add(Provider())
+
 
 class Polygon(factory.DjangoModelFactory):
     class Meta:
         model = models.Polygon
 
-    creation_date = datetime.datetime(2018, 8, 1, tzinfo=pytz.utc)
-    deletion_date = None
     label = ""
     geom = district10
     properties = {}
 
 
-class Telemetry(factory.DjangoModelFactory):
+class EventRecord(factory.DjangoModelFactory):
     class Meta:
-        model = models.Telemetry
+        model = models.EventRecord
 
     device = factory.SubFactory(Device)
-    provider = uuid.UUID("a19cdb1e-1342-413b-8e89-db802b2f83f6")
-    timestamp = datetime.datetime(2018, 8, 1, tzinfo=pytz.utc)
-    status = factory.Iterator(choice[0] for choice in enums.DEVICE_STATUS_CHOICES)
-    point = factory.LazyFunction(get_random_point)
+    timestamp = factory.Sequence(
+        lambda n: datetime.datetime(2018, 8, 1, tzinfo=pytz.utc)
+        + datetime.timedelta(seconds=n)
+    )
+    point = factory.LazyAttribute(
+        lambda f: geos.Point(
+            f.properties["telemetry"]["gps"]["lng"],
+            f.properties["telemetry"]["gps"]["lat"],
+        )
+    )
+    saved_at = datetime.datetime(2018, 8, 1, 1, tzinfo=pytz.utc)
+    event_type = factory.Iterator(c.name for c in enums.EVENT_TYPE)
     properties = factory.Dict(
         {
-            "gsm": factory.Dict(
+            "trip_id": None,
+            "telemetry": factory.Dict(
                 {
-                    "timestamp": datetime.datetime(2018, 8, 1, tzinfo=pytz.utc),
-                    "operator": "operator_test",
-                    "signal": 0.5,
+                    "timestamp": 1_325_376_000_000,
+                    "gps": factory.Dict(
+                        {
+                            "lat": 0.0,
+                            "lng": 3.0,
+                            "altitude": 30.0,
+                            "heading": 245.2,
+                            "speed": 32.3,
+                            "accuracy": 2.0,
+                        }
+                    ),
+                    "battery_pct": 0.5,
                 }
             ),
-            "gps": factory.Dict(
-                {
-                    "timestamp": datetime.datetime(2018, 8, 1, tzinfo=pytz.utc),
-                    "accuracy": 1,
-                    "course": 235.62,
-                    "speed": 119.15,
-                }
-            ),
-            "vehicle_state": factory.Dict(
-                {
-                    "speed": 1.0,
-                    "acceleration": [1.0, 1.0, 1.0],
-                    "odometer": 5000,
-                    "driver_present": True,
-                }
-            ),
-            "energy": factory.Dict({"cruise_range": 12000, "autonomy": 0.69}),
         }
     )
+
+
+class ProviderStatusChange(factory.DictFactory):
+    """Excepted status change from the provider API."""
+
+    provider_id = factory.Faker("uuid4")
+    provider_name = factory.Iterator(["Lime", "BlueLA", "Metro Bike"])
+    device_id = factory.Faker("uuid4")
+    vehicle_id = factory.Sequence(str)
+    vehicle_type = factory.Iterator(c.name for c in enums.DEVICE_CATEGORY)
+    propulsion_type = factory.Iterator([c.name] for c in enums.DEVICE_PROPULSION)
+    event_type = factory.Iterator(c.name for c in enums.DEVICE_STATUS)
+    event_type_reason = factory.Iterator(enums.PROVIDER_REASON_TO_AGENCY_EVENT)
+    event_time = 1_529_968_782_421
+    event_location = factory.Dict(
+        {
+            "type": "Feature",
+            "properties": factory.Dict({"timestamp": 1_529_968_782_421}),
+            "geometry": factory.Dict(
+                {
+                    "type": "Point",
+                    "coordinates": factory.LazyFunction(
+                        lambda: list(utils.get_random_point(district10).coords)
+                    ),
+                }
+            ),
+        }
+    )
+    battery_pct = 0.5
+    associated_trip = None
+
+
+class ProviderStatusChangesBody(factory.DictFactory):
+    """Expected response from the provider API."""
+
+    version = "0.3.0"
+    data = factory.Dict(
+        {"status_changes": factory.List([factory.SubFactory(ProviderStatusChange)])}
+    )
+    links = factory.Dict({"next": None})
